<?php if (!defined('PmWiki')) exit();
/**
 * LaTeX Rendering Class - Calling function
 * Copyright (C) 2003  Benjamin Zeiss <zeiss@math.uni-goettingen.de>
 *
 * Modified to work with PmWiki v2
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * --------------------------------------------------------------------
 * @author Benjamin Zeiss <zeiss@math.uni-goettingen.de>
 * @version v0.12
 * @package latexrender
 * Revised by Steve Mayer
 * Modified for PmWiki by John Rankin/Tyrel Newton
 * Patched to work under PmWiki 2.2 and to support remote latexrender server
 * This file can be included in many PHP programs by using something like
 * (see example.php to see how it can be used)
 * 		include_once('/full_path_here_to/latexrender/latex.php');
 * 		$text_to_be_converted=latex_content($text_to_be_converted);
 * $text_to_be_converted will then contain the link to the appropriate image
 * or an error code as follows (the 500 values can be altered in class.latexrender.php):
 * 	0 OK
 * 	1 Formula longer than 500 characters
 * 	2 Includes a blacklisted tag
 * 	3 (Not used) Latex rendering failed
 * 	4 Cannot create DVI file
 * 	5 Picture larger than 500 x 500 followed by x x y dimensions
 * 	6 Cannot copy image to pictures directory
 */
$latexrender_path = "$FarmD/pub/latexrender";
SDV($FarmPubDirUrl, $PubDirUrl);
$latexrender_path_http = "$FarmPubDirUrl/latexrender";
$EquationLabels = array();
$HTMLStylesFmt['latexr'] = "img.math { vertical-align: text-bottom; }
span.eqcount { float: right; font-style: italic; margin-left: 1em; }";
Markup('latexr', '<directives', '/\{\$(.*?)\$\}/', "latexrHelper");
Markup('latexdm', '<latexr', '/^(\s?-+&gt;|=\|)?\s*\{\$(.*?)\$\}\s*$/',
  "latexdmHelper");
Markup('latexeq', '<latexdm', '/\{\$\s*([A-Za-z][-.:\\w]*)\s*,(.*?)\$\}/',
  "latexeqHelper");
Markup('latexar', '<latexeq',
  '/\{\$:([^=]+)=(\d+):(?:\s*([A-Za-z][-.:\\w]*)\s*,)?\s*(.*?)\$\}/',
  "latexarHelper");
Markup('displaymath','>include','/(\{\$(?:\s*[A-Za-z][^,]*,)?)\n(.*?)\n(\$\})/s',
  "displaymathHelper");
Markup('eqnarray', '>include',
  '/\{\$([*#]|[A-Za-z][^,]*,)?(([^\n]*\$\}\n\{\$[^\n]*)+)\$\}/s',
  "LatexREqnArray");
Markup('eqref', '>latexr', '/(E[Qq])\(([A-Za-z][-.:\\w]*)\)/', "LatexEqRef");

function latexrHelper($m) {
  return LatexRWiki('math',$m[1]);
}
function latexdmHelper($m) {
  return $m[1].LatexRWiki('displaymath',$m[2]);
}
function latexeqHelper($m) {
  return LatexEqNum($m[1]).LatexRWiki('equation',$m[2]);
}
function latexarHelper($m) {
  return ($m[3] ? LatexEqNumber(LatexEqRefLookup($m[3]),$m[3]) : '').
    LatexRWiki(LatexArrayNum($m[1],$m[2],$m[3]),$m[4]);
}
function displaymathHelper($m) {
  return $m[1].str_replace("\n",'&#10;',$m[2]).$m[3];
}  
function LatexEqRefLookup($label) {
    global $EquationLabels;
    foreach ($EquationLabels as $k => $v)
        if ($v == $label) return $k;
        else $c = $k;
    return $c+1;
}
function LatexEqRef($m) {
    global $EquationPrefix;
    SDV($EquationPrefix, 'equation');
    $txt = $EquationPrefix;
    $kind  = $m[1];
    $label = $m[2];
    if ($kind=='Eq') $txt[0] = strtoupper($txt[0]);
    $ref = "$txt&nbsp;[[#$label | (\$Number)]]";
    return str_replace('$Number',LatexEqRefLookup($label),$ref);
}
function LatexEqCount($inc) {
    static $EquationCount;
    $EquationCount += $inc;
    return $EquationCount;
}
function LatexEqNumber($number, $label) {
    global $LatexrenderNumberFmt;
    SDV($LatexrenderNumberFmt, "<span class='eqcount' title='\$Label'>".
        "<a name='\$Label' id='\$Label'></a>(\$Number)</span>");
    return Keep(str_replace('$Number', $number,
                str_replace('$Label',  $label, $LatexrenderNumberFmt)), 'L');
}
function LatexEqNum($label) {
    global $EquationLabels;
    $EquationCount = LatexEqCount(1);
    $EquationLabels[$EquationCount] = $label;
    return LatexEqNumber($EquationCount, $label);
}
function LatexArrayNum($class, $inc, $label) {
    global $EquationLabels;
    $EquationCount = LatexEqCount($inc);
    $index = $EquationCount;
    if ($class=='align') $index -= $inc;
    $EquationLabels[$EquationCount] = ($label ? $label : "$class$index");
    return "$class$index";
}
function LatexREqnArray($m) {
    $star = $m[1]; $text = $m[2];
    if (strlen($star)>1) { $anch = $star; $star = '#'; }
    else $anch = '';
    $al = ($star=='*') ? 'center' : 'right';
    $st = ($star=='*') ? 'alignstar' : (($star=='#') ? 'subequations' : 'align');
    foreach(explode("\$}\n{\$",$text) as $t)
      $out[] = strstr($t,'&amp;') ? $t :
      (preg_match('/\s(=|&[gl]t;|\\\(?:[gln]eq?|ll|gg|approx|simeq|doteq|sim|equiv|cong|propto))\s/',
          $t,$m) ? str_replace($m[0]," &amp;$m[1] ",$t) : " &amp; $t"); 
    if ($star=='*')     $count = 0;
    elseif ($star=='#') $count = 1;
    else {
        $count = count($out);
        foreach($out as $o) 
          if (strstr($o,'\nonumber') || strstr($o,'\notag')) $count--;
    }
    return "%$al%".($count ? '{$'.":$st=$count: $anch".implode('\\\\&#10;',$out).'$}' : 
                             LatexRWiki($st, implode('\\\\&#10;',$out)));
}


function LatexRWiki($class, $text) {
  global $latexrender_local, $latexrender_path_http, $latexrender_server_url, 
         $LatexrenderUrlFmt, $LatexrenderImageFmt;
  SDV($latexrender_local, true);
  if ($latexrender_local) return LatexRLocal($class, $text);
  SDV($latexrender_server_url, $latexrender_path_http);
  $lr_server = $latexrender_server_url.'/scripts/latex2img.php';
  SDV($LatexrenderUrlFmt, '<img class="$LinkClass" src="$LinkUrl?'.
        '$LinkTitle?mode=$LinkMode" title="$LinkTitle" align=absmiddle />');
  $latex_formula = str_replace(array("\r","\n",'"'),
        array("&#13;","&#10;",'&quot;'), trim($text));
  $oclass = preg_replace('/\d+$/','',$class);
  if ($LatexrenderImageFmt == 'svg') $class .= '&amp;'.$LatexrenderImageFmt;
  return Keep(str_replace(array('$LinkUrl', '$LinkTitle',   '$LinkClass', '$LinkMode'),
                          array( $lr_server, $latex_formula, $oclass,      $class), 
                          $LatexrenderUrlFmt), 'L');
}

function LatexRLocal($class, $text) {
// -------------------------------------------------------------------------
// adjust this to match your system configuration
    global $latexrender_path, $latexrender_path_http, $LatexrenderLatexPath, $LatexrenderDvipsPath,
        $LatexrenderDvisvgmPath, $LatexrenderConvertPath, $LatexrenderIdentifyPath, $LatexrenderUrlFmt, $LatexrenderImageFmt;
    SDV($latexrender_path,  "/home/domain_name/public_html/latexrender");
    SDV($latexrender_path_http, "/latexrender");
    SDV($LatexrenderUrlFmt, "<img class='\$LinkClass' src='\$LinkUrl' ".
        "title=\"\$LinkTitle\" alt=\"\$LinkTitle\" align=absmiddle />");
    SDV($LatexrenderImageFmt,     "gif");
    SDV($LatexrenderLatexPath,    "/usr/bin/latex");
    SDV($LatexrenderDvipsPath,    "/usr/bin/dvips");
    SDV($LatexrenderDvisvgmPath,  "/usr/bin/dvisvgm");
    SDV($LatexrenderConvertPath,  "/usr/bin/convert");
    SDV($LatexrenderIdentifyPath, "/usr/bin/identify");
// -------------------------------------------------------------------------
    
// include main rendering class, note that it must be in the scripts directory
    include_once($latexrender_path."/scripts/class.latexrender.php");
    
    $latex = new LatexRender($latexrender_path."/pictures",
        $latexrender_path_http."/pictures", $latexrender_path."/tmp");

//  set location of required binaries
    $latex->_latex_path = $LatexrenderLatexPath;
    $latex->_dvips_path = $LatexrenderDvipsPath;
    $latex->_dvisvgm_path  = $LatexrenderDvisvgmPath;
    $latex->_convert_path  = $LatexrenderConvertPath;
    $latex->_identify_path = $LatexrenderIdentifyPath;
    $latex_formula = trim($text);
    $latex->_image_format = $LatexrenderImageFmt;
    $url = $latex->getFormulaURL($latex_formula,$class);
    $class = preg_replace('/\d+$/','',$class);

    $alt_latex_formula = str_replace(array("\r","\n",'"'),
        array("&#13;","&#10;",'&quot;'), $latex_formula);

   return ($url) ? Keep(str_replace(array('$LinkUrl', '$LinkTitle', '$LinkClass'),
        array($url, $alt_latex_formula, $class), $LatexrenderUrlFmt), 'L') :
            "[Unparseable or potentially dangerous latex formula. ".
            "Error $latex->_errorcode $latex->_errorextra]";
             
}

?>
